@@ -1,14 +1,13 @@
-if (getRversion() >= "2.15.1") utils::globalVariables(c("counter", "Prediction", "input.data", "old.d", "old.d2", "surv", "event", "n.risk", "part"))
+utils::globalVariables(c("old.d2", "surv", "event", "n.risk", "part"))
 
-DynNom.cph <- function(model, data,
-                       clevel = 0.95, covariate = c("slider", "numeric"),
-                       ptype = c("st", "1-st")) {
+DynNom.cph <- function(model, data, clevel = 0.95, m.summary = c("raw", "formatted"),
+                       covariate = c("slider", "numeric"), ptype = c("st", "1-st")) {
 
   data <- data.frame(data)
   model <- update(model,x=T, y=T, surv=T)
 
   if(dim(model$y)[2]==3)
-    stop("Error in model syntax: models with start-stop time is not supported")       
+    stop("Error in model syntax: models with start-stop time is not supported")
 
   if (length(dim(data)) > 2)
     stop("Error in data format: dataframe format required")
@@ -26,7 +25,7 @@ DynNom.cph <- function(model, data,
   for(i in 1:length(model$Design$name)){
     if(model$Design$assume[i]!="interaction") vars[i] <- as.character(model$Design$name[i]) else vars[i]="inter"
   }
-  
+
   vars <- subset(vars,vars!="inter")
   vars <- as.character(c(model$terms[[2]],vars))
 
@@ -41,6 +40,7 @@ DynNom.cph <- function(model, data,
   dim.terms <- length(vars)
 
   covariate <- match.arg(covariate)
+  m.summary <- match.arg(m.summary)
   ptype <- match.arg(ptype)
   input.data <- NULL
   old.d <- NULL
@@ -61,7 +61,7 @@ DynNom.cph <- function(model, data,
       ),
       mainPanel(tabsetPanel(id = "tabs",
                             tabPanel("Estimated S(t)", plotOutput("plot")),
-                            tabPanel("Predicted Survival", plotOutput("plot2")),
+                            tabPanel("Predicted Survival", plotlyOutput("plot2")),
                             tabPanel("Numerical Summary", verbatimTextOutput("data.pred")),
                             tabPanel("Model Summary", verbatimTextOutput("summary"))
       )
@@ -172,15 +172,17 @@ DynNom.cph <- function(model, data,
             if (nn > 2){
               slide.bars <- list(lapply(1:(nn - 2), function(j) {
                 sliderInput(paste("numeric", j, sep = ""), i.numeric[j, 1],
-                            min = as.integer(min(na.omit(data[, as.numeric(i.numeric[j, 3])]))),
-                            max = as.integer(max(na.omit(data[, as.numeric(i.numeric[j, 3])]))) + 1,
-                            value = as.integer(mean(na.omit(data[, as.numeric(i.numeric[j, 3])]))))
+                            min = floor(min(na.omit(data[, as.numeric(i.numeric[j, 3])]))),
+                            max = ceiling(max(na.omit(data[, as.numeric(i.numeric[j, 3])]))),
+                            value = mean(na.omit(data[, as.numeric(i.numeric[j, 3])]))
+                            )
               }), br(), checkboxInput("times", "Predicted Survival at this Follow Up:"),
               conditionalPanel(condition = "input.times == true",
                                sliderInput(paste("numeric", (nn - 1), sep = ""), i.numeric[(nn - 1), 1],
-                                           min = as.integer(min(na.omit(model$y[,1]))),
-                                           max = as.integer(max(na.omit(model$y[,1]))) + 1,
-                                           value = as.integer(mean(na.omit(model$y[,1]))))
+                                           min = floor(min(na.omit(model$y[,1]))),
+                                           max = ceiling(max(na.omit(model$y[,1]))),
+                                           value = mean(na.omit(model$y[,1]))
+                                           )
               ))
             }
 
@@ -188,9 +190,10 @@ DynNom.cph <- function(model, data,
               slide.bars <- list(br(), checkboxInput("times", "Predicted Survival at this Follow Up:"),
                                  conditionalPanel(condition = "input.times == true",
                                                   sliderInput(paste("numeric", (nn - 1), sep = ""), i.numeric[(nn - 1), 1],
-                                                              min = as.integer(min(na.omit(model$y[,1]))),
-                                                              max = as.integer(max(na.omit(model$y[,1]))) + 1,
-                                                              value = as.integer(mean(na.omit(model$y[,1]))))
+                                                              min = floor(min(na.omit(model$y[,1]))),
+                                                              max = ceiling(max(na.omit(model$y[,1]))),
+                                                              value = mean(na.omit(model$y[,1]))
+                                                              )
                                  ))
             }
           }
@@ -199,17 +202,17 @@ DynNom.cph <- function(model, data,
             if (nn > 2){
               slide.bars <- list(lapply(1:(nn - 2), function(j) {
                 numericInput(paste("numeric", j, sep = ""), i.numeric[j, 1],
-                             value = as.integer(mean(na.omit(data[, as.numeric(i.numeric[j, 3])]))))
+                             value = round(mean(na.omit(data[, as.numeric(i.numeric[j, 3])]))))
               }), br(), checkboxInput("times", "Predicted Survival at this Follow Up:"),
               conditionalPanel(condition = "input.times == true",
                                numericInput(paste("numeric", (nn - 1), sep = ""), i.numeric[(nn - 1), 1],
-                                            value = as.integer(mean(na.omit(model$y[,1]))))))
+                                            value = round(mean(na.omit(model$y[,1]))))))
             }
             if (nn == 2){
               slide.bars <- list(br(), checkboxInput("times", "Predicted Survival at this Follow Up:"),
                                  conditionalPanel(condition = "input.times == true",
                                                   numericInput(paste("numeric", (nn - 1), sep = ""), i.numeric[(nn - 1), 1],
-                                                               value = as.integer(mean(na.omit(model$y[,1]))))))
+                                                               value = round(mean(na.omit(model$y[,1]))))))
             }
           }
           do.call(tagList, slide.bars)
@@ -218,10 +221,11 @@ DynNom.cph <- function(model, data,
 
       a <- 0
       new.d <- reactive({
+        input$add
         if (nf > 0) {
           input.f <- vector("list", nf)
           for (i in 1:nf) {
-            input.f[[i]] <- local({
+            input.f[[i]] <- isolate({
               input[[paste("factor", i, sep = "")]]
             })
             names(input.f)[i] <- i.factor[i, 1]
@@ -230,7 +234,7 @@ DynNom.cph <- function(model, data,
         if (nn > 1) {
           input.n <- vector("list", (nn - 1))
           for (i in 1:(nn - 1)) {
-            input.n[[i]] <- local({
+            input.n[[i]] <- isolate({
               input[[paste("numeric", i, sep = "")]]
             })
             names(input.n)[i] <- i.numeric[i, 1]
@@ -256,7 +260,9 @@ DynNom.cph <- function(model, data,
         if (a > 0) {
           wher <- match(names(out), names(input.data))
           out2 <- cbind(out[wher], NO=input$add)
-          input.data <<- rbind(input.data, out2)
+          if (isTRUE(compare(old.d, out)) == FALSE) {
+            input.data <<- rbind(input.data, out2)
+          }
         }
         a <<- a + 1
         out
@@ -332,65 +338,59 @@ DynNom.cph <- function(model, data,
               i <- i+1
             }
           }
-          sub.fit1 <- subset(as.data.frame(summary(fit1)[2:8]), strata == nam)
+          sub.fit1 <- subset(as.data.frame(summary(fit1)[c(2:4,6:7)]), strata == nam)
           return(sub.fit1)
         })
       }
 
       dat.p <- reactive({
         if (isTRUE(compare(old.d2, new.d())) == FALSE) {
-          s.frame <- isolate({
-            if (is.error(try(survfit(model,new.d())))==T){
-              stop("new dataset has strata levels not found in the original")
+          if (is.error(try(survfit(model,new.d())))==T){
+            stop("new dataset has strata levels not found in the original")
+          }
+          fit1 <- survfit(model, newdata = new.d())
+          if (n.strata == 0) {
+            sff <- as.data.frame(summary(fit1)[c(2:4,6:7)])
+            sff <- cbind(sff, event=1-sff$surv, part = b)
+            if (sff$time[1] != 0){
+              sff2 <- sff[1, ]
+              sff2[1, ] <- NA
+              sff2$time[1] <- 0
+              sff2$n.risk[1] <- sum(model$n)
+              sff2$surv[1] <- 1
+              sff2$event[1] <- 0
+              sff2$part[1] <- sff$part[1]
+              s.f <- rbind(sff2, sff)
+            } else {
+              s.f <- sff
             }
-            fit1 <- survfit(model, newdata = new.d())
-            if (n.strata == 0) {
-              sff <- as.data.frame(summary(fit1)[2:8])
-              sff <- cbind(sff, event=1-sff[[5]], part = b)
-              if (sff$time[1] != 0){
-                sff2 <- sff[1, ]
-                sff2[1, ] <- NA
-                sff2$time[1] <- 0
-                sff2$n.risk[1] <- sum(model$n)
-                sff2$surv[1] <- 1
-                sff2$event[1] <- 0
-                sff2$part[1] <- sff$part[1]
-                s.f <- rbind(sff2, sff)
-              } else {
-                s.f <- sff
-              }
+          }
+          if (n.strata > 0) {
+            sff <- cbind(sub.fit1(), part = b)
+            sff <- cbind(sff, event=1-sff$surv)
+            if (sff$time[1] != 0) {
+              sff2 <- sff[1, ]
+              sff2[1, ] <- NA
+              sff2$time[1] <- 0
+              sff2$n.risk[1] <- sff[1,2]
+              sff2$surv[1] <- 1
+              sff2$event[1] <- 0
+              sff2$part[1] <- sff$part[1]
+              s.f <- rbind(sff2, sff)
+            } else {
+              s.f <- sff
             }
-            if (n.strata > 0) {
-              sff <- cbind(sub.fit1(), part = b)
-              sff <- cbind(sff, event=1-sff[[6]])
-              if (sff$time[1] != 0) {
-                sff2 <- sff[1, ]
-                sff2[1, ] <- NA
-                sff2$time[1] <- 0
-                sff2$n.risk[1] <- sff[1,2]
-                sff2$surv[1] <- 1
-                sff2$event[1] <- 0
-                sff2$part[1] <- sff$part[1]
-                s.f <- rbind(sff2, sff)
-              } else {
-                s.f <- sff
-              }
-              s.f$n.risk <- s.f$n.risk/s.f$n.risk[1]
-            }
-            if (dim(s.f)[1] < 3) {
-              St <<- FALSE
-              stop("Error in data structure: There is not enough data in the current strata level")
-            }
-            s.fr <<- rbind(s.fr, s.f)
-            old.d2 <<- new.d()
-            b <<- b + 1
-            s.fr
-          })
-        } else {
-          s.frame <- isolate({
-            s.fr
-          })
+            s.f$n.risk <- s.f$n.risk/s.f$n.risk[1]
+          }
+          if (dim(s.f)[1] < 3) {
+            St <<- FALSE
+            stop("Error in data structure: There is not enough data in the current strata level")
+          }
+          s.fr <<- rbind(s.fr, s.f)
+          old.d2 <<- new.d()
+          b <<- b + 1
         }
+        s.frame <- s.fr
       })
 
       coll=c(1:20)
@@ -398,46 +398,41 @@ DynNom.cph <- function(model, data,
         if (St == TRUE) {
           if (input$add == 0)
             return(NULL)
-
           if (input$add > 0) {
             if (input$trans == TRUE) {
               if (ptype == "st") {
-                pl <- isolate({
-                  p2 <- ggplot(data = dat.p())
-                  p2 <- p2 + geom_step(aes(x = time, y = surv, alpha = n.risk,group = part), color = coll[dat.p()$part])
-                  p2 <- p2 + ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05)
-                  p2 <- p2 + labs(title = "Estimated Survival Probability", x = "Follow Up Time", y = "S(t)") + theme_bw()
-                  p2 <- p2 + theme(text = element_text(face = "bold", size = 14), legend.position = "none")
-                })
+                pl <- ggplot(data = dat.p()) +
+                  geom_step(aes(x = time, y = surv, alpha = n.risk,group = part), color = coll[dat.p()$part]) +
+                  ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05) +
+                  labs(title = "Estimated Survival Probability", x = "Follow Up Time", y = "S(t)") + theme_bw() +
+                  theme(text = element_text(face = "bold", size = 12), legend.position = "none",
+                      plot.title = element_text(hjust = .5))
               }
               if (ptype == "1-st") {
-                pl <- isolate({
-                  p2 <- ggplot(data = dat.p())
-                  p2 <- p2 + geom_step(aes(x = time, y = event, alpha = n.risk,group = part), color = coll[dat.p()$part])
-                  p2 <- p2 + ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05)
-                  p2 <- p2 + labs(title = "Estimated Probability", x = "Follow Up Time", y = "F(t)")
-                  p2 <- p2 + theme_bw() + theme(text = element_text(face = "bold", size = 14), legend.position = "none")
-                })
+                pl <- ggplot(data = dat.p()) +
+                  geom_step(aes(x = time, y = event, alpha = n.risk,group = part), color = coll[dat.p()$part]) +
+                  ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05) +
+                  labs(title = "Estimated Probability", x = "Follow Up Time", y = "F(t)") + theme_bw() +
+                  theme(text = element_text(face = "bold", size = 12), legend.position = "none",
+                        plot.title = element_text(hjust = .5))
               }
             }
             if (input$trans == FALSE) {
               if (ptype == "st") {
-                pl <- isolate({
-                  p2 <- ggplot(data = dat.p())
-                  p2 <- p2 + geom_step(aes(x = time, y = surv, group = part), color = coll[dat.p()$part])
-                  p2 <- p2 + ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05)
-                  p2 <- p2 + labs(title = "Estimated Survival Probability", x = "Follow Up Time", y = "S(t)") + theme_bw()
-                  p2 <- p2 + theme(text = element_text(face = "bold", size = 14), legend.position = "none")
-                })
+                pl <- ggplot(data = dat.p()) +
+                  geom_step(aes(x = time, y = surv, group = part), color = coll[dat.p()$part]) +
+                  ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05) +
+                  labs(title = "Estimated Survival Probability", x = "Follow Up Time", y = "S(t)") + theme_bw() +
+                  theme(text = element_text(face = "bold", size = 12), legend.position = "none",
+                        plot.title = element_text(hjust = .5))
               }
               if (ptype == "1-st") {
-                pl <- isolate({
-                  p2 <- ggplot(data = dat.p())
-                  p2 <- p2 + geom_step(aes(x = time, y = event,group = part), color = coll[dat.p()$part])
-                  p2 <- p2 + ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05)
-                  p2 <- p2 + labs(title = "Estimated Probability", x = "Follow Up Time", y = "F(t)")
-                  p2 <- p2 + theme_bw() + theme(text = element_text(face = "bold", size = 14), legend.position = "none")
-                })
+                pl <- ggplot(data = dat.p()) +
+                  geom_step(aes(x = time, y = event,group = part), color = coll[dat.p()$part]) +
+                  ylim(0, 1) + xlim(0, max(dat.p()$time) * 1.05) +
+                  labs(title = "Estimated Probability", x = "Follow Up Time", y = "F(t)") + theme_bw() +
+                  theme(text = element_text(face = "bold", size = 12), legend.position = "none",
+                        plot.title = element_text(hjust = .5))
               }
             }
           }
@@ -449,61 +444,66 @@ DynNom.cph <- function(model, data,
         }
       })
 
-      output$plot2 <- renderPlot({
+      output$plot2 <- renderPlotly({
         if (input$add == 0)
           return(NULL)
-
-        isolate({
-          if (is.null(new.d()))   return(NULL)
-          if (dim(na.omit(data2()))[1]==0 ) return(NULL)
-          lim <- c(0, 1)
-          yli <- c(0 - 0.5, 10 + 0.5)
-          if (dim(input.data)[1] > 11)
-            yli <- c(dim(input.data)[1] - 11.5, dim(input.data)[1] - 0.5)
-          p <- ggplot(data = data2()[!is.na(data2()$Prediction),], aes(x = Prediction, y = 0:(sum(counter) - 1)))
-          p <- p + geom_point(size = 4, colour = coll[data2()$count[!is.na(data2()$Prediction)]], shape = 15)  
-          p <- p + ylim(yli[1], yli[2]) + coord_cartesian(xlim = lim)
-          p <- p + geom_errorbarh(xmax = data2()$Upper.bound[!is.na(data2()$Prediction)], xmin = data2()$Lower.bound[!is.na(data2()$Prediction)],
-                                  size = 1.45, height = 0.4, colour = coll[data2()$count[!is.na(data2()$Prediction)]])         
-          if (ptype == "st") {
-            p <- p + labs(title = paste(clevel * 100, "% ", "Confidence Interval for Survival Probability", sep = ""),
-                          x = "Survival Probability", y = NULL)
-          }
-          if (ptype == "1-st") {
-            p <- p + labs(title = paste(clevel * 100, "% ", "Confidence Interval for F(t)", sep = ""),
-                          x = "Probability", y = NULL)
-          }
-          p <- p + theme_bw() + theme(axis.text.y = element_blank(), text = element_text(face = "bold", size = 14))
-          print(p)
-        })
-        data2()
+        if (is.null(new.d()))   return(NULL)
+        if (dim(na.omit(data2()))[1]==0 ) return(NULL)
+        lim <- c(0, 1)
+        PredictNO <- 0:(sum(data2()$counter) - 1)
+        in.d <- data.frame(input.data[-1,-dim(input.data)[2]])
+        xx=matrix(paste(names(in.d), ": ",t(in.d), sep=""), ncol=dim(in.d)[1])
+        Covariates=apply(xx,2,paste,collapse="<br />")
+        yli <- c(0 - 0.5, 10 + 0.5)
+        if (dim(input.data)[1] > 11)
+          yli <- c(dim(input.data)[1] - 11.5, dim(input.data)[1] - 0.5)
+        p <- ggplot(data = data2()[!is.na(data2()$Prediction),], aes(x = Prediction, y =  PredictNO,
+                                                                     text = Covariates, label = Prediction, label2 = Lower.bound, label3=Upper.bound))
+        p <- p + geom_point(size = 2, colour = coll[data2()$count[!is.na(data2()$Prediction)]], shape = 15)
+        p <- p + ylim(yli[1], yli[2]) + coord_cartesian(xlim = lim)
+        p <- p + geom_errorbarh(xmax = data2()$Upper.bound[!is.na(data2()$Prediction)], xmin = data2()$Lower.bound[!is.na(data2()$Prediction)],
+                                size = 1.45, height = 0.4, colour = coll[data2()$count[!is.na(data2()$Prediction)]])
+        if (ptype == "st") {
+          p <- p + labs(title = paste(clevel * 100, "% ", "Confidence Interval for Survival Probability", sep = ""),
+                        x = "Survival Probability", y = NULL)
+        }
+        if (ptype == "1-st") {
+          p <- p + labs(title = paste(clevel * 100, "% ", "Confidence Interval for F(t)", sep = ""),
+                        x = "Probability", y = NULL)
+        }
+        p <- p + theme_bw() + theme(axis.text.y = element_blank(), text = element_text(face = "bold", size = 10))
+        gp=ggplotly(p, tooltip = c("text","label","label2","label3"))
+        dat.p()
+        gp
       })
 
       output$data.pred <- renderPrint({
         if (input$add > 0) {
-          isolate({
-            if (nrow(data2() > 0)) {
-              di <- ncol(input.data)
-              data.p <- merge(input.data[-1, ], data2()[1:5], by="NO")
-              data.p <- data.p[, !(colnames(data.p) %in% c("NO", "counter"))]
-              data.p$Prediction[is.na(data.p$Prediction)] <- "Not"
-              data.p$Lower.bound[is.na(data.p$Lower.bound)] <- "IN"
-              data.p$Upper.bound[is.na(data.p$Upper.bound)] <- "RANGE"
-              stargazer(data.p, summary = FALSE, type = "text")
-            }
-          })
+          if (nrow(data2() > 0)) {
+            di <- ncol(input.data)
+            data.p <- merge(input.data[-1, ], data2()[1:5], by="NO")
+            data.p <- data.p[, !(colnames(data.p) %in% c("NO", "counter"))]
+            data.p$Prediction[is.na(data.p$Prediction)] <- "Not"
+            data.p$Lower.bound[is.na(data.p$Lower.bound)] <- "IN"
+            data.p$Upper.bound[is.na(data.p$Upper.bound)] <- "RANGE"
+            stargazer(data.p, summary = FALSE, type = "text")
+          }
         }
       })
 
       output$summary <- renderPrint({
-        if (is.null(model$coef) == T){
+        if (m.summary == "raw"){
           print(model)
-        } else {
-          coef.c <- exp(model$coef)
-          ci.c <- exp(suppressMessages(confint(model, level = clevel)))
-          stargazer(model,model$stats, coef = list(coef.c), ci.custom = list(ci.c), p.auto = F,
-                    type = "text", omit.stat = c("LL", "ser", "f"), ci = TRUE, ci.level = clevel,
-                    single.row = TRUE, title = paste("Cox model:", model$call[2], sep = " "))
+        } else{
+          if (is.null(model$coef) == T){
+            print(model)
+          } else {
+            coef.c <- exp(model$coef)
+            ci.c <- exp(suppressMessages(confint(model, level = clevel)))
+            stargazer(model,model$stats, coef = list(coef.c), ci.custom = list(ci.c), p.auto = F,
+                      type = "text", omit.stat = c("LL", "ser", "f"), ci = TRUE, ci.level = clevel,
+                      single.row = TRUE, title = paste("Cox model:", model$call[2], sep = " "))
+          }
         }
       })
     }
